//
//  ConflictsTableViewController.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 08/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "ConflictsTableViewController.h"


@implementation ConflictsTableViewController

@synthesize delegate, conflicts, conflictsData, managedObjectContext, persistentStoreCoordinator, conflictsTableView;

- (void)dealloc
{
    [delegate release];
    [managedObjectContext release];
    [persistentStoreCoordinator release];
    [conflicts release];
    [conflictsData release];
    [conflictsTableView release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil delegate:(id<ConflictsTableViewControllerProtocol>)aDelegateOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        self.delegate = aDelegateOrNil;
    }
    
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    self.managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    self.persistentStoreCoordinator = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] persistentStoreCoordinator];
}

- (void)viewDidUnload
{
    [conflictsTableView release];
    conflictsTableView = nil;
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    [fetchRequest setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    [fetchRequest setPredicate:[NSPredicate predicateWithFormat:@"state!=%i", 0]];
    NSError *error = nil;
    self.conflicts = [[self managedObjectContext] executeFetchRequest:fetchRequest error:&error];
    if (error) {
        NSLog(@"Error retrieving SyncTasks with state != 0: %@", [error localizedDescription]);
    }
    NSLog(@"Fetched %i conflicts", [conflicts count]);
    [fetchRequest release];
    
    self.conflictsData = [[NSMutableArray alloc] init];
    for (NSInteger i = 0; i < [conflicts count]; i++) {
        NSMutableDictionary *aConflictData = [[NSMutableDictionary alloc] init];
        NSManagedObjectID *objectID = [persistentStoreCoordinator managedObjectIDForURIRepresentation:[NSURL URLWithString:[(SyncTask *)[conflicts objectAtIndex:i] objectURI]]];
        if ([objectID entity] == [NSEntityDescription entityForName:@"Category" inManagedObjectContext:self.managedObjectContext]) {
            [aConflictData setValue:NSLocalizedString(@"Category", @"Category") forKey:@"entity"];
        } else {
            [aConflictData setValue:NSLocalizedString(@"Track", @"Track") forKey:@"entity"];
        }
        NSManagedObject *anObject = [[self managedObjectContext] objectWithID:objectID];
        [aConflictData setValue:[anObject valueForKey:@"name"] forKey:@"name"];
        [conflictsData addObject:aConflictData];
    }
    NSLog(@"Set %i conflictsData", [conflictsData count]);
    [[self conflictsTableView] reloadData];
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return [conflicts count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return 2;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [NSString stringWithFormat:@"%@: %@", [[conflictsData objectAtIndex:section] valueForKey:@"entity"], [[conflictsData objectAtIndex:section] valueForKey:@"name"]];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    
    switch (indexPath.row) {
        case 0:
            [[cell textLabel] setText:NSLocalizedString(@"Mobile data", @"Mobile data - Conflict")];
            if ([[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue] == 2) {
                [cell setAccessoryType:UITableViewCellAccessoryCheckmark];
            }
            break;
        case 1:
            [[cell textLabel] setText:NSLocalizedString(@"Web data", @"Web data - Conflict")];
            if ([[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue] == 3) {
                [cell setAccessoryType:UITableViewCellAccessoryCheckmark];
            }
            break;
        default:
            break;
    }
    
    return cell;
}

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
    
    if (indexPath.row == 0) {
        if ([[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue] != 2) {
            NSLog(@"Seleccionamos row 0 y != 2");
            [(SyncTask *)[conflicts objectAtIndex:indexPath.section] setState:[NSNumber numberWithInt:2]];
            NSLog(@"SyncTask state: %i", [[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue]);
            [[tableView cellForRowAtIndexPath:indexPath] setAccessoryType:UITableViewCellAccessoryCheckmark];
            [[tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:1 inSection:indexPath.section]] setAccessoryType:UITableViewCellAccessoryNone];
        } else {
            NSLog(@"Seleccionamos row 0 y else");
            [(SyncTask *)[conflicts objectAtIndex:indexPath.section] setState:[NSNumber numberWithInt:1]];
            NSLog(@"SyncTask state: %i", [[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue]);
            [[tableView cellForRowAtIndexPath:indexPath] setAccessoryType:UITableViewCellAccessoryNone];
        }        
    } else {
        if ([[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue] != 3) {
            NSLog(@"Seleccionamos row 1 y != 3");
            [(SyncTask *)[conflicts objectAtIndex:indexPath.section] setState:[NSNumber numberWithInt:3]];
            NSLog(@"SyncTask state: %i", [[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue]);
            [[tableView cellForRowAtIndexPath:indexPath] setAccessoryType:UITableViewCellAccessoryCheckmark];
            [[tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:indexPath.section]] setAccessoryType:UITableViewCellAccessoryNone];
        } else {
            NSLog(@"Seleccionamos row 1 y else");
            [(SyncTask *)[conflicts objectAtIndex:indexPath.section] setState:[NSNumber numberWithInt:1]];
            NSLog(@"SyncTask state: %i", [[(SyncTask *)[conflicts objectAtIndex:indexPath.section] state] intValue]);
            [[tableView cellForRowAtIndexPath:indexPath] setAccessoryType:UITableViewCellAccessoryNone];
        }
    }
//    [tableView reloadSections:[NSIndexSet indexSetWithIndex:indexPath.section] withRowAnimation:NO];
}

#pragma mark - Actions

- (IBAction)saveChanges:(id) sender
{
    NSLog(@"Saving changes...");
    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        NSLog(@"Error saving conflicts: %@", [error localizedDescription]);
    }
    if (delegate && [delegate respondsToSelector:@selector(conflictsTableViewControllerDidEndEditing)]) {
        [delegate conflictsTableViewControllerDidEndEditing];
    }    
    [self dismissModalViewControllerAnimated:YES];
}

- (IBAction)cancel:(id)sender
{
    [managedObjectContext undo];
    if (delegate && [delegate respondsToSelector:@selector(conflictsTableViewControllerDidCancelEditing)]) {
        [delegate conflictsTableViewControllerDidCancelEditing];
    }
    [self dismissModalViewControllerAnimated:YES];
}

@end
