//
//  EditTrackViewController.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 02/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "EditTrackViewController.h"


@implementation EditTrackViewController

@synthesize delegate, scrollView, nameTextField, ppmTextField, noteTextView, categoryLabel, activeField, scrolViewContentsHeight, managedObjectContext, theTrack;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (id)initEditWithTrack:(Track *)aTrack delegate:(id<EditTrackViewProtocol>)aDelegate
{
    self = [super initWithNibName:@"EditTrackView" bundle:nil];
    if (self) {
        self.delegate = aDelegate;
        self.theTrack = aTrack;
    }
    
    return self;
}

- (void)dealloc
{
    [scrollView release];
    [nameTextField release];
    [ppmTextField release];
    [noteTextView release];
    [managedObjectContext release];
    [theTrack release];
    [categoryLabel release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    [self registerKeyboardNotifications];
    
    managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    
    nameTextField.text = [theTrack name];
    ppmTextField.text = ([[theTrack ppm] intValue] > 0) ? [NSString stringWithFormat:@"%i", [[theTrack ppm] integerValue]] : @"";
    noteTextView.text = [theTrack note];
    categoryLabel.text = [[theTrack category] name];
    [nameTextField becomeFirstResponder];
}

- (void)viewDidUnload
{
    [scrollView release];
    scrollView = nil;
    [nameTextField release];
    nameTextField = nil;
    [ppmTextField release];
    ppmTextField = nil;
    [noteTextView release];
    noteTextView = nil;
    [categoryLabel release];
    categoryLabel = nil;
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Actions

- (IBAction)save
{
    if ([self validate]) {
        [theTrack setName:[nameTextField text]];
        [theTrack setPpm:[NSNumber numberWithInt:[[ppmTextField text] integerValue]]];
        [theTrack setNote:[noteTextView text]];
        [theTrack setUpdatedTimestamp];
        [theTrack writeKMLData];
        [(SyncManager *)[SyncManager sharedSyncManager] registerSyncTaskType:SyncTaskModify forManagedObject:theTrack];
        NSError *error;
        if (![managedObjectContext save:&error]) {
            NSLog(@"ERROR - EditTrackViewController: theTrack couldn't be saved \n %@", [error description]);
            UIAlertView *alertView = [[UIAlertView alloc] 
                                      initWithTitle:NSLocalizedString(@"Error saving data", @"Alert title - error") 
                                      message:NSLocalizedString(@"The data couldn't be saved", @"Alert description - error") 
                                      delegate:nil 
                                      cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                      otherButtonTitles:nil];
            [alertView show];
        } else {
            if ([delegate respondsToSelector:@selector(editTrackViewDidEndEditing:)]) {
                [delegate editTrackViewDidEndEditing:self];
            }
            [self dismissModalViewControllerAnimated:YES];
        }
    }
}

- (IBAction)cancel
{
    [self dismissModalViewControllerAnimated:YES];
}

- (IBAction)done:(id)sender
{
    if (sender == nameTextField) {
        [ppmTextField becomeFirstResponder];
    } else if (sender == ppmTextField) {
        [noteTextView becomeFirstResponder];
    } else {
        [noteTextView resignFirstResponder];
    }
}

- (IBAction)showSelectCategoryTable:(id)sender {
    SelectCategoryTableViewController *selectCategoryTableViewController = [[SelectCategoryTableViewController alloc] initWithTrack:theTrack delegate:self];
    
    [self presentModalViewController:selectCategoryTableViewController animated:YES];
}

- (BOOL)validate
{
    BOOL result = YES;
    if ([[nameTextField text] isEqualToString:@""]) {
        UIAlertView *alertView = [[UIAlertView alloc] 
                                  initWithTitle:NSLocalizedString(@"Validation error", @"Alert title - validation error") 
                                  message:NSLocalizedString(@"The track must have a name", @"Alert description - name error") 
                                  delegate:nil 
                                  cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                  otherButtonTitles:nil];
        [alertView show];
        result = NO;
    } else if ([[ppmTextField text] integerValue] < 0 || [[ppmTextField text] integerValue] > 250) {
        UIAlertView *alertView = [[UIAlertView alloc] 
                                  initWithTitle:NSLocalizedString(@"Validation error", @"Alert title - validation error") 
                                  message:NSLocalizedString(@"PPM value can not be less than 0 or greater than 250", @"Alert description - ppm error") 
                                  delegate:nil 
                                  cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                  otherButtonTitles:nil];
        [alertView show];
        result = NO;
    }
    
    return result;
}

#pragma mark - SelectCategoryTable methods

- (void)selectCategoryTableViewControllerEndEditingWithCategoryName:(NSString *)categoryName
{
    NSLog(@"selectCategoryChanged");
    categoryLabel.text = categoryName;
}

#pragma mark - Keyboard methods

- (void)registerKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWasShown:) name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillBeHidden:) name:UIKeyboardDidHideNotification object:nil];
}

- (void)keyboardWasShown:(NSNotification *)aNotification
{
    NSLog(@"keyboardWasShown");
    NSDictionary* info = [aNotification userInfo];
    CGSize kbSize = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
    [[self scrollView] setFrame:CGRectMake(0, scrollView.frame.origin.y, scrollView.frame.size.width, scrollView.frame.size.height - kbSize.height)];
    
    scrollViewContentsHeight = 80.0;
    for (NSInteger i = 0; i < [[scrollView subviews] count]; i++) {
        UIView *aView = (UIView *)[[scrollView subviews] objectAtIndex:i];
        scrollViewContentsHeight += aView.frame.size.height;
    }
    [[self scrollView] setContentSize:CGSizeMake(scrollView.frame.size.width, scrollViewContentsHeight)];
    if (activeField == noteTextView) {
        CGPoint moveToPoint = CGPointMake(0.0, scrollView.contentSize.height - scrollView.frame.size.height);
        [scrollView setContentOffset:moveToPoint animated:YES];
    }
    NSLog(@"scrollView frame: (%f, %f, %f, %f)", scrollView.frame.origin.x, scrollView.frame.origin.y, scrollView.frame.size.width, scrollView.frame.size.height);
    NSLog(@"scrollView contentSize: (%f, %f)", scrollView.contentSize.width, scrollView.contentSize.height);

}

- (void)keyboardWillBeHidden:(NSNotification *)aNotification
{
    [scrollView setFrame:CGRectMake(0.0, scrollView.frame.origin.y, scrollView.frame.size.width, scrollView.superview.frame.size.height-44)];
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    activeField = textField;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    activeField = nil;
}

@end
