//
//  NewTrackViewController.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 24/04/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "NewTrackViewController.h"

#define kStartTimePause @"startTimePause"
#define kTotalPauseTimeInterval @"totalPauseTimeInterval"
#define kCurrentIntervalPauseTimeInterval @"currentIntervalPauseTimeInterval"


@implementation NewTrackViewController

@synthesize totalTimeLabel;
@synthesize intervalTimeLabel;
@synthesize startButton;
@synthesize stopButton;
@synthesize managedObjectContext;
@synthesize sharedTrackLocationManager;
@synthesize tracking;
@synthesize timer;
@synthesize track;
@synthesize currentPlacemark;
@synthesize pauseTimes;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [totalTimeLabel release];
    [intervalTimeLabel release];
    [startButton release];
    [stopButton release];
    [managedObjectContext release];
    [timer release];
    [track release];
    [currentPlacemark release];
    [pauseTimes release];
    
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    self.tracking = NO;
    
    self.managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    self.sharedTrackLocationManager = (TrackLocationManager *)[TrackLocationManager sharedTrackLocationManager];
    [[self sharedTrackLocationManager] setDelegate:self];

}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
//    [totalTimeLabel release];    
//    self.totalTimeLabel = nil;
//    [intervalTimeLabel release];    
//    self.intervalTimeLabel = nil;
//    [startButton release];
//    self.startButton = nil;
//    [stopButton release];
//    self.stopButton = nil;
    
}

- (void)viewDidAppear:(BOOL)animated
{
    [[[self view] viewWithTag:1] setHidden:![sharedTrackLocationManager registerPlacemarks]];
    [intervalTimeLabel setHidden:![sharedTrackLocationManager registerPlacemarks]];
    [super viewDidAppear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Actions


/*
 * Inicia l'enregistrament d'un nou recorregut:
 *   - Estableix el temps d'inici
 *   - Activa el temporitzador
 *   - Indica que s'enregistren les actualitzacions del GPS
 *   - Estableix la marca d'inici del recorregut
 */
- (IBAction)startTracking:(id)sender
{
    NSLog(@"Tracking started");
    // Mostra el botó de pausar i Parar
    [startButton setHidden:YES];
    [stopButton setHidden:NO];
    
    // Establim a nivell d'aplicació que s'està enregistrant.
    [(CorreAppDelegate *) [[UIApplication sharedApplication] delegate] setTracking:YES];
    
    tracking = YES;
    
    NSLog(@"Distance filter: %i - Mark's distance: %i", [sharedTrackLocationManager distanceFilter], [sharedTrackLocationManager distanceBetweenPlacemarks]);
    NSLog(@"CL - Distance filter: %.2f - Mark's distance: %i", [[sharedTrackLocationManager locationManager] distanceFilter], [sharedTrackLocationManager distanceBetweenPlacemarks]);
    [self logRecords];
        
    // Inicialitzem un nou recorregut amb la posició d'inici
    Place *startPlace = [NSEntityDescription insertNewObjectForEntityForName:@"Place" inManagedObjectContext:self.managedObjectContext];
    [startPlace setValuesFromLocation:[[sharedTrackLocationManager locationManager] location]];
    track = (Track *)[NSEntityDescription insertNewObjectForEntityForName:@"Track" inManagedObjectContext:managedObjectContext];
    [track setDate:[NSDate date]];
    [track addPlacesObject:startPlace];
    [track setName:NSLocalizedString(@"New Track", @"New Track")];
    
    // Si s'enregistren les marques inicialitzem la primera
    if ([sharedTrackLocationManager registerPlacemarks]) {
        currentPlacemark = (Placemark *) [NSEntityDescription insertNewObjectForEntityForName:@"Placemark" inManagedObjectContext:self.managedObjectContext];
        [currentPlacemark setStartTime:[NSDate date]];
        [currentPlacemark setPlace:startPlace];
        [currentPlacemark setName:NSLocalizedString(@"Placemark", @"Placemark")];
        [track addPlacemarksObject:currentPlacemark];
    }
    startPlace = nil;    
    
    // Iniciem el temporitzador
    [pauseTimes setValue:[NSNumber numberWithDouble:0.0] forKey:kTotalPauseTimeInterval];
    [pauseTimes setValue:[NSNumber numberWithDouble:0.0] forKey:kCurrentIntervalPauseTimeInterval];
    timer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(updateTimer) userInfo:nil repeats:YES];
    
    [timer fire];
}

/*
 * Para l'enregistrament d'un recorregut
 *   - Atura el temporitzador
 *   - Atura l'enregistrament 
 *   - Estableix la marca de fi de recorregut
 */
- (IBAction)stopTracking:(id)sender
{
    NSLog(@"Tracking stopped");
    // Parem el temporitzador

    [self updateTimer];
    [timer invalidate];
    
    Place *stopPlace = (Place *)[NSEntityDescription insertNewObjectForEntityForName:@"Place" inManagedObjectContext:self.managedObjectContext];
    [stopPlace setValuesFromLocation:[[sharedTrackLocationManager locationManager] location]];
    // Actualitzem els valors de temps i 
    [track addPlacesObject:stopPlace];
    [track setTime:[NSNumber numberWithDouble:-[[track date] timeIntervalSinceNow]]];
    // Assignem la última categoria seleccionada
    NSPersistentStoreCoordinator *persistentStoreCoordinator = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] persistentStoreCoordinator];
    NSURL *categoryURI = [NSURL URLWithString:[[NSUserDefaults standardUserDefaults] objectForKey:@"LastCategorySelected"]];
    NSLog(@"Category URI: %@", [categoryURI absoluteString]);
    NSManagedObjectID *categoryID = [persistentStoreCoordinator managedObjectIDForURIRepresentation:categoryURI];
    [track setCategory:(Category *) [managedObjectContext objectWithID:categoryID]];
    // Actualitzem el valor de l'última marca
    if ([sharedTrackLocationManager registerPlacemarks]) {
        CLLocation *oldLocation = [[CLLocation alloc] initWithLatitude:[[[currentPlacemark place] latitude] floatValue] longitude:[[[currentPlacemark place] longitude] floatValue]];
        [self updateIntervalValuesWithNewLocation:[[sharedTrackLocationManager locationManager] location] fromLocation:oldLocation place:stopPlace];
        [oldLocation release];
    }
    stopPlace = nil;
    
    // Desem els canvis del managedObjectContext
    [track setUpdatedTimestamp];
    [track writeKMLData];
    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        NSLog(@"Error saving data: %@", [error localizedDescription]);
        NSLog(@"Error: %@", [error description]);
    }
    [(SyncManager *)[SyncManager sharedSyncManager] registerSyncTaskType:SyncTaskCreate forManagedObject:track];
    NSLog(@"ID de la track: %@", [[track objectID] URIRepresentation]);
    // Mostrem la vista per modificar la informació de la ruta
    EditTrackViewController *editTrackViewController = [[EditTrackViewController alloc] initEditWithTrack:track delegate:self];
    [self presentModalViewController:editTrackViewController animated:YES];
            
    [stopButton setHidden:YES];
    [startButton setHidden:NO];
    
    // Llevem l'indicador que estem enregistrant a nivell d'aplicació
    [(CorreAppDelegate *) [[UIApplication sharedApplication] delegate] setTracking:NO];
    tracking = NO;
    
}

/*
 * Funció cridada pel temporitzador cada interval definit
 *   - S'actualitza les etiquetes amb el temps.
 */
- (void)updateTimer
{
    totalTimeLabel.text = [NSString stringWithFormat:@"%@", [self formatTimeInterval: -[[track date] timeIntervalSinceNow]]];
    intervalTimeLabel.text = [NSString stringWithFormat:@"%@", [self formatTimeInterval: -[[currentPlacemark startTime] timeIntervalSinceNow]]];
    
}

#pragma mark Core Location Delegate methods

/*
 * Mètode delegat cridat quan locationManager actualitza una posició.
 *
 */
- (void)trackLocationManager:(TrackLocationManager *)theTrackLocationManager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation
{
    if (newLocation.horizontalAccuracy < 0) {
        return;
    }
    if (tracking) {
        NSLog(@"Location manager upated: From (%f, %f) to (%f, %f), distance: %f", oldLocation.coordinate.longitude, oldLocation.coordinate.latitude, newLocation.coordinate.longitude, newLocation.coordinate.latitude, [newLocation distanceFromLocation:oldLocation]);
        Place *newPlace = (Place *)[NSEntityDescription insertNewObjectForEntityForName:@"Place" inManagedObjectContext:self.managedObjectContext];
        [newPlace setValuesFromLocation:newLocation];
        [self updateTrackValuesWithNewLocation:newLocation fromLocation:oldLocation place:newPlace];
        [self updateIntervalValuesWithNewLocation:newLocation fromLocation:oldLocation place:newPlace];
        newPlace = nil;
    } else {
        NSLog(@"No estem enregistrant");
    }
}  

#pragma mark Location Helpers

/*
 * Actualitza els valors emmagatzemats del recorregut:
 *    - Distància acumulada
 *    - Valors màxims i mínims
 */
- (void)updateTrackValuesWithNewLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation place:(Place *)aPlace
{
    [track addPlacesObject:aPlace];
    // Si el valor d'exactitud horitzontal és menor que 0 ens indica que és erroni i omitim la resta d'operacions.
    if (newLocation.horizontalAccuracy < 0) {
        return;
    }
    // Distància total recorreguda
    [track setDistance:[NSNumber numberWithFloat:[[track distance] floatValue] + [newLocation distanceFromLocation:oldLocation]]];
    // Altitud màxima
    if ([[track maxAltitude] floatValue] < [newLocation altitude]) {
        [track setMaxAltitude:[NSNumber numberWithFloat:[newLocation altitude]]];
    }
    // Altitud mínima
    if ([[track minAltitude] floatValue] > [newLocation altitude]) {
        [track setMinAltitude:[NSNumber numberWithFloat:[newLocation altitude]]];
    }
    // Velocitat màxima
    float splitSpeed = [newLocation distanceFromLocation:oldLocation] / -[[oldLocation timestamp] timeIntervalSinceDate:[newLocation timestamp]];
    NSLog(@"Split speed: %f m/s - %f km/h", splitSpeed, splitSpeed * 3.6);
    if ([[track maxSpeed] floatValue] < splitSpeed) {
        [track setMaxSpeed:[NSNumber numberWithFloat:splitSpeed]];
    }
}

/*
 * Actualitza els valors emmagatzemats de l'últim interval:
 *    - Distància acumulada
 *    - Valors màxims i mínims
 */
- (void)updateIntervalValuesWithNewLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation place:(Place *)aPlace
{
    // Distància total recorreguda de l'interval
    [currentPlacemark setDistance:[NSNumber numberWithFloat:[[currentPlacemark distance] floatValue] + [newLocation distanceFromLocation:oldLocation]]];
    // Altitud màxima
    if ([[currentPlacemark maxAltitude] floatValue] < [newLocation altitude]) {
        [currentPlacemark setMaxAltitude:[NSNumber numberWithFloat:[newLocation altitude]]];
    }
    // Altitud mínima
    if ([[currentPlacemark minAltitude] floatValue] > [newLocation altitude]) {
        [currentPlacemark setMinAltitude:[NSNumber numberWithFloat:[newLocation altitude]]];
    }
    // Velocitat màxima
    float splitSpeed = [newLocation distanceFromLocation:oldLocation] / -[[oldLocation timestamp] timeIntervalSinceDate:[newLocation timestamp]];
    if ([[currentPlacemark maxSpeed] floatValue] < splitSpeed) {
        [currentPlacemark setMaxSpeed:[NSNumber numberWithFloat:splitSpeed]];
    }
    // Emmagatzemem la posició de la marca
    [currentPlacemark setPlace:aPlace];
    // Emmagatzemem el temps
    [currentPlacemark setTime:[NSNumber numberWithDouble:-[[currentPlacemark startTime] timeIntervalSinceNow]]];
    NSLog(@"%i >= %i", (NSInteger)[[currentPlacemark distance] floatValue], [sharedTrackLocationManager distanceBetweenPlacemarks]);
    // Si l'actual interval ha arribat a la distància distanceBetweenPlacemarks creem un nou interval
    if ((NSInteger)[[currentPlacemark distance] floatValue] >= [sharedTrackLocationManager distanceBetweenPlacemarks]) {
        // Creem un nou interval
        currentPlacemark = nil;
        currentPlacemark = (Placemark *) [NSEntityDescription insertNewObjectForEntityForName:@"Placemark" inManagedObjectContext:self.managedObjectContext];
        [currentPlacemark setStartTime:[NSDate date]];
        [currentPlacemark setName:NSLocalizedString(@"Placemark", @"Placemark")];
        [track addPlacemarksObject:currentPlacemark];
    }
}

#pragma mark EditTrackView protocol method

- (void)editTrackViewDidEndEditing:(EditTrackViewController *)editTrackViewController
{
    UIAlertView *alertView = [[UIAlertView alloc] 
                              initWithTitle:NSLocalizedString(@"Data saved", @"Alert view - New track saved") 
                              message:NSLocalizedString(@"Track values updated correctly", @"Alert view - Track saved description") 
                              delegate:nil 
                              cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                              otherButtonTitles:nil];
    [alertView show];
    track = nil;
}

#pragma mark Helpers

/*
 * Calcula l'interval expressat en milisegons a segons, minuts i hores i l'aplica el format h:mm:ss
 */ 
- (NSString*)formatTimeInterval:(NSTimeInterval) theInterval
{
    int hours = 0;
    int minutes = 0;
    int seconds = 0;
    if (theInterval >= 60*60) {
        hours = round(theInterval / 60*60);
    }
    if (theInterval - hours*60*60 >= 60) {
        minutes = floor((theInterval - hours * 60 * 60) / 60);
    }
    seconds = floor(theInterval - hours*60*60 - minutes*60);
    return [NSString stringWithFormat:@"%i:%02i:%02i", hours, minutes, seconds];
}

/*
 * Registra el moment quan es pausa el cronòmetre
 */
- (void)registerPauseTimer
{
    [pauseTimes setValue:[NSDate date] forKey:kStartTimePause];
}


/*
 * Afegeix el tems que ha estat el cronòmetre pausat:
 *    - Al total del temps
 *    - Al temps pausat de l'últim interval
 */
- (void)resumePauseTimer
{
    NSTimeInterval totalPauseTimeInterval = [[pauseTimes valueForKey:kTotalPauseTimeInterval] doubleValue] - [[pauseTimes valueForKey:kStartTimePause] timeIntervalSinceNow];
    NSTimeInterval currentIntervalPauseTimeInterval = [[pauseTimes valueForKey:kCurrentIntervalPauseTimeInterval] doubleValue] - [[pauseTimes valueForKey:kStartTimePause] timeIntervalSinceNow];
    [pauseTimes setValue:[NSNumber numberWithDouble:totalPauseTimeInterval] forKey:kTotalPauseTimeInterval];
    [pauseTimes setValue:[NSNumber numberWithDouble:currentIntervalPauseTimeInterval] forKey:kCurrentIntervalPauseTimeInterval];
}


- (void)logRecords
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSArray *sortDescriptors = [NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES] autorelease]];
    [request setSortDescriptors:sortDescriptors];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Track" inManagedObjectContext:self.managedObjectContext];
    [request setEntity:entity];
    NSError *error;
    NSArray *data = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    NSLog(@"Recorreguts guardats: %i", [data count]);
    NSLog(@"%@", [[data lastObject] description]);
    NSLog(@"%@", [[[data lastObject] placemarks] description]);
    [request release];
    [data release];
}

@end
