//
//  SettingsViewController.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 12/04/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SettingsViewController.h"
#import "SetupPickerViewController.h"

#define sectionsCount 3
#define userSettingsRegion 0
#define registerSettingsRegion 1
#define categorySettingsRegion 2

#define kUpdatedDateKey @"UpdateDate"
#define kNicknameKey @"Nickname"
#define kPasswordKey @"Password"
#define kPlacemarksKey @"Placemarks"
#define kMarksDistanceKey @"MarksDistance"
#define kAccuracyKey @"Accuracy"

NSString * const kOptionName = @"OptionName";
NSString * const kOptionValue = @"OptionValue";

@implementation SettingsViewController

@synthesize managedObjectContext;

@synthesize userNicknameTextField;
@synthesize userPasswordTextField;

@synthesize categoriesCell;

@synthesize togglePlacemarksSwitch;

@synthesize userSettings;
@synthesize registerSettings;

@synthesize optionsDistanceMarks;
@synthesize optionsAccuracy;

@synthesize userDefaults;

#pragma mark - Initialization and destruction

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [userDefaults release];
    
    [userNicknameTextField release];
    [userPasswordTextField release];
    
    [categoriesCell release];
    
    [setupDistanceMarks release];
    [setupAccuracy release];
    [setupCategories release];

    [userSettings release];    
    [registerSettings release];
    
    [optionsDistanceMarks release];
    [optionsAccuracy release];
    
    [managedObjectContext release];
    
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - Getters and setters

// Els mètodes d'accès i assignació de les opcions s'implementen sense syntetize per carregar les dades quan realment es necessita.
// D'aquesta forma només s'empra memòria quan es necessita.

// Accessor i assignador de la propietat setupDistanceMarks
- (SetupPickerViewController *)setupDistanceMarks
{
    if (setupDistanceMarks == nil) {
        NSInteger rowSelected = [self searchRowForValue:[userDefaults integerForKey:kMarksDistanceKey] inArray:optionsDistanceMarks];
        NSMutableDictionary *interfaceValues = [self setInterfaceValuesWithOptions:optionsDistanceMarks
                                                            helpText:NSLocalizedString(@"Set the desired distance between placemarks", @"Help text - Distance between placemarks")
                                                            settingText:NSLocalizedString(@"Distance:", @"Picker's label")
                                                        returnKey:@"setupDistanceMarks"];
        
        setupDistanceMarks = [[SetupPickerViewController alloc] initWithInterfaceData:interfaceValues rowSelected:rowSelected delegate:self];       
    }
    
    return setupDistanceMarks;
}

- (void)setSetupDistanceMarks:(SetupPickerViewController *)aSetupDistanceMarks
{
    self.setupDistanceMarks = aSetupDistanceMarks;
}


// Accessor i assignador de la propietat setupAccuracy
- (SetupPickerViewController *)setupAccuracy
{
    if (setupAccuracy == nil) {
        NSInteger rowSelected = [self searchRowForValue:[userDefaults integerForKey:kAccuracyKey] inArray:self.optionsAccuracy];
        NSMutableDictionary *interfaceValues = [self setInterfaceValuesWithOptions:optionsAccuracy
                                                                   helpText:NSLocalizedString(@"Set the desired distance that the GPS updates the position", @"Help text - Accuracy")
                                                                settingText:NSLocalizedString(@"Distance:", @"Picker's label")
                                                                  returnKey:@"setupAccuracy"];
        
        setupAccuracy = [[SetupPickerViewController alloc] initWithInterfaceData:interfaceValues rowSelected:rowSelected delegate:self];       
    }
    
    return setupAccuracy;
}

- (void)setSetupAccuracy:(SetupPickerViewController *)aSetupAccuracy
{
    self.setupAccuracy = aSetupAccuracy;
}

// Accessor de la propietat setupCategories
- (SetupTableViewController *)setupCategories
{
    if (setupCategories == nil) {
        setupCategories = [[SetupTableViewController alloc] initWithNibName:@"SetupTableView" bundle:nil];
        setupCategories.hidesBottomBarWhenPushed = YES;
    }
    
    return setupCategories;
    
}

#pragma mark - View lifecycle

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView
{
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{    
    [super viewDidLoad];
    
    self.managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    
    userDefaults = [NSUserDefaults standardUserDefaults];
    [self setupOptions];
    
    userSettings = [[NSMutableArray alloc] init];     
    [userSettings addObject:[[self newCellWithLabel:NSLocalizedString(@"Nickname", @"Nickname") reuseIdentifier:@"NicknameCell" textField:userNicknameTextField] autorelease]];
    [userSettings addObject:[[self newCellWithLabel:NSLocalizedString(@"Password", @"Password") reuseIdentifier:@"PasswordCell" textField:userPasswordTextField] autorelease]];
    
    [self setupRegisterSettings];
    
    NSLog(@"%i", [userDefaults integerForKey:kMarksDistanceKey]);
    NSLog(@"%i", [userDefaults integerForKey:kAccuracyKey]);
    [[[registerSettings objectAtIndex:1] detailTextLabel] setText:[[optionsDistanceMarks objectAtIndex:[self searchRowForValue:[userDefaults integerForKey:kMarksDistanceKey] inArray:optionsDistanceMarks]] objectForKey:kOptionName]];
    [[[registerSettings objectAtIndex:2] detailTextLabel] setText:[[optionsAccuracy objectAtIndex:[self searchRowForValue:[userDefaults integerForKey:kAccuracyKey] inArray:optionsAccuracy]] objectForKey:kOptionName]];
    
    categoriesCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"CategoriesCell"];
    [[categoriesCell textLabel] setText:NSLocalizedString(@"Categories", @"Categories")];
    [categoriesCell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];

    userNicknameTextField.text = [userDefaults objectForKey:kNicknameKey];
    userPasswordTextField.text = [userDefaults objectForKey:kPasswordKey];
    
    togglePlacemarksSwitch.on = [userDefaults boolForKey:kPlacemarksKey];
}


- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.userNicknameTextField = nil;
    self.userPasswordTextField = nil;
    self.togglePlacemarksSwitch = nil;
    
    [setupDistanceMarks release];
    setupDistanceMarks = nil;
    [setupAccuracy release];
    setupAccuracy = nil;
    [setupCategories release];
    setupCategories = nil;
}

- (void)viewDidAppear:(BOOL)animated
{
    // Si actualment s'està enregistrant es bloquegen les preferències relatives a l'enregistrament.
    [togglePlacemarksSwitch setEnabled:![(CorreAppDelegate *) [[UIApplication sharedApplication] delegate] tracking]];
    UITableViewCellSelectionStyle cellSelectionStyle = ([(CorreAppDelegate *) [[UIApplication sharedApplication] delegate] tracking]) ? UITableViewCellSelectionStyleNone : UITableViewCellSelectionStyleBlue;
    [[registerSettings objectAtIndex:1] setSelectionStyle:cellSelectionStyle];
    [[registerSettings objectAtIndex:2] setSelectionStyle:cellSelectionStyle];
    
    [super viewDidAppear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return sectionsCount;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    switch (section) {
        case userSettingsRegion:
            return [userSettings count];
            break;
        case registerSettingsRegion:
            return [registerSettings count];
            break;
        case categorySettingsRegion:
            return 1;
            break;
        default:
            return 0;
            break;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
        
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        [cell setEditingAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
    }
    
    // Configure the cell...
    
    switch (indexPath.section) {
        case userSettingsRegion:
            return [self.userSettings objectAtIndex:indexPath.row];
            break;
        case registerSettingsRegion:
            return [registerSettings objectAtIndex:indexPath.row];
            break;
        case categorySettingsRegion:
            return self.categoriesCell;
            break;
        default:
            [[cell textLabel] setText:NSLocalizedString(@"Unknown",@"Default table value")];
            break;
    }
    
    return cell;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    switch (section) {
        case userSettingsRegion:
            return NSLocalizedString(@"User settings", @"User settings' table header");
            break;
        case registerSettingsRegion:
            return NSLocalizedString(@"Register settings", @"Register settings' table header");
            break;
        case categorySettingsRegion:
            return NSLocalizedString(@"Category settings", @"Category settings' table header");
            break;
        default:
            return NSLocalizedString(@"Unknown", @"Default table value");
            break;
    }
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
    
    if (indexPath.section == registerSettingsRegion) {
        if (indexPath.row == 1 && ![(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] tracking]) {
            [[self setupDistanceMarks] setRowSelected:[self searchRowForValue:[userDefaults integerForKey:kMarksDistanceKey] inArray:optionsDistanceMarks]];
            [self presentModalViewController:self.setupDistanceMarks animated:YES];
        } else if (indexPath.row == 2 && ![(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] tracking]) {
            [[self setupAccuracy] setRowSelected:[self searchRowForValue:[userDefaults integerForKey:kAccuracyKey] inArray:optionsAccuracy]];
            [self presentModalViewController:self.setupAccuracy animated:YES];
        }
    } else if (indexPath.section == categorySettingsRegion) {
        [[self navigationController] pushViewController:self.setupCategories animated:YES];
    }
}

#pragma mark - Alert view delegate methods

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSString *buttonClicked = [alertView buttonTitleAtIndex:buttonIndex];
    if (buttonClicked == NSLocalizedString(@"Yes", @"Yes")) {
        [(SyncManager *)[SyncManager sharedSyncManager] clearAllSyncTasks];
        [(SyncManager *)[SyncManager sharedSyncManager] createAllNewSyncTasks];
    }
}

#pragma mark - Picker view controller delegate

- (void)setupPickerViewController:(SetupPickerViewController *)setupPickerViewController didFinishSetupWithInfo:(NSDictionary *)setupInfo
{
    if ([setupInfo objectForKey:@"ReturnKey"] == @"setupDistanceMarks") {
        [[[registerSettings objectAtIndex:1] detailTextLabel] setText:[[setupInfo objectForKey:@"Value"] objectForKey:kOptionName]];
        [userDefaults setInteger:[[[setupInfo objectForKey:@"Value"] objectForKey:kOptionValue] integerValue] forKey:kMarksDistanceKey];
    } else if ([setupInfo objectForKey:@"ReturnKey"] == @"setupAccuracy") {
        [[[registerSettings objectAtIndex:2] detailTextLabel] setText:[[setupInfo objectForKey:@"Value"] objectForKey:kOptionName]];
        [userDefaults setInteger:[[[setupInfo objectForKey:@"Value"] objectForKey:kOptionValue] integerValue] forKey:kAccuracyKey];

    }
    [userDefaults synchronize];
}

#pragma mark - Actions

- (IBAction)hideKeyboard:(UITextField *) sender
{
    for (UITableViewCell *cell in userSettings) {
        [cell resignFirstResponder];
    }
}

- (IBAction)didEndEditing:(UITextField *) sender
{
    NSLog(@"DidEndEditing");
    if ([sender isEqual:userNicknameTextField]) {
        NSLog(@"Nick");
        NSString *oldNickname = [userDefaults objectForKey:kNicknameKey];
        [userDefaults setObject:userNicknameTextField.text forKey:kNicknameKey];
        if (oldNickname != userNicknameTextField.text) {
            UIAlertView *alertView = [[UIAlertView alloc]
                                      initWithTitle:NSLocalizedString(@"Nickname change", @"Nickname change") 
                                      message:NSLocalizedString(@"You changed your nickname. Do you want to send all your data on next synchronization?", @"Alert view message - Resend data") 
                                      delegate:self 
                                      cancelButtonTitle:NSLocalizedString(@"No", @"No") 
                                      otherButtonTitles:NSLocalizedString(@"Yes", @"Yes"), nil];
            [alertView show];
            [alertView release];
        }
        oldNickname = nil;
    } else if ([sender isEqual:userPasswordTextField]) {
        NSLog(@"Password");
        [userDefaults setObject:userPasswordTextField.text forKey:kPasswordKey];
    }
    [userDefaults synchronize];
}

- (IBAction)setPlaceMarksChanged:(id)sender
{
    togglePlacemarksSwitch.on ? NSLog(@"Sí") : NSLog(@"No");
    [userDefaults setBool:togglePlacemarksSwitch.on forKey:kPlacemarksKey];
    [userDefaults synchronize];
}

- (NSMutableDictionary *)setInterfaceValuesWithOptions:(NSArray *)options helpText:(NSString *)aHelpText settingText:(NSString *)aSettingText returnKey:(NSString *)aReturnKey
{
    NSMutableDictionary *interfaceValues = [NSDictionary dictionaryWithObjectsAndKeys:
                                      aHelpText,@"helpText",
                                      aSettingText,@"settingText",
                                      options,@"Options",
                                      aReturnKey,@"ReturnKey",
                                      nil
                                      ];
    return interfaceValues;
}

- (UITableViewCell *)newCellWithLabel:(NSString *)aLabel reuseIdentifier:(NSString *)aReuseIdentifier textField:(UITextField *)aTextField
{
    UITableViewCell *aCell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:aReuseIdentifier];
    [[aCell textLabel] setText:aLabel];
    [aCell setSelectionStyle:UITableViewCellEditingStyleNone];
    [aCell insertSubview:aTextField atIndex:[[aCell subviews] count]];
    CGRect cellFrame = [aCell frame];
    [[aCell viewWithTag:1] setFrame:CGRectMake(cellFrame.size.width/2, 1, (cellFrame.size.width / 2) - 14, cellFrame.size.height)];
    return aCell;
}

- (void)setupRegisterSettings
{
    registerSettings = [[NSMutableArray alloc] init];
    
    // Cel·la Placemarsk
    UITableViewCell *aCell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"SetPlacemarksCell"] autorelease];
    [[aCell textLabel] setText:NSLocalizedString(@"Placemarks", @"Placemarks")];
    [aCell insertSubview:togglePlacemarksSwitch atIndex:1];
    [[aCell viewWithTag:1] setFrame:CGRectMake((aCell.frame.size.width - togglePlacemarksSwitch.frame.size.width - 15) , (aCell.frame.size.height/2 - (togglePlacemarksSwitch.frame.size.height)/2 + 1), togglePlacemarksSwitch.frame.size.width, togglePlacemarksSwitch.frame.size.height)];
    [aCell setSelectionStyle:UITableViewCellEditingStyleNone];
    [registerSettings addObject:aCell];
    aCell = nil;
    
    // Cel·la Marks Distance
    aCell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"MarksDistanceCell"] autorelease];
    [[aCell textLabel] setText:NSLocalizedString(@"Mark's distance" , @"Mark's distance")];
    [registerSettings addObject:aCell];
    aCell = nil;
    
    // Cel·la Accuracy
    aCell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"AccuracyCell"] autorelease];
    [[aCell textLabel] setText:NSLocalizedString(@"Accuracy" , @"Accuracy")];
    [registerSettings addObject:aCell];
    aCell = nil;
}

- (NSInteger)searchRowForValue:(NSInteger)aValue inArray:(NSArray *)anArray
{
    NSInteger index = -1;
    NSInteger i = 0;
    while (i < [anArray count] && index < 0) {
        if (aValue == [[[anArray objectAtIndex:i] objectForKey:kOptionValue] integerValue]) {
            index = i;
        }
        i++;
    }
    
    return index;
}

- (void)setupOptions
{
    
    
    optionsDistanceMarks = [[NSMutableArray alloc] init];
    [optionsDistanceMarks addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"100 meters", @"100 meters"), kOptionName, [NSNumber numberWithInteger:100], kOptionValue, nil]];
    [optionsDistanceMarks addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"500 meters", @"500 meters"), kOptionName, [NSNumber numberWithInteger:500], kOptionValue, nil]];
    [optionsDistanceMarks addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"1 kilometer", @"1 kilometer"), kOptionName, [NSNumber numberWithInteger:1000], kOptionValue, nil]];
    [optionsDistanceMarks addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"5 kilometers", @"5 kilometers"), kOptionName, [NSNumber numberWithInteger:5000], kOptionValue, nil]];
    [optionsDistanceMarks addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"10 kilometers", @"10 kilometers"), kOptionName, [NSNumber numberWithInteger:10000], kOptionValue, nil]];
    
    optionsAccuracy = [[NSMutableArray alloc] init];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"5 meters", @"5 meters"), kOptionName, [NSNumber numberWithInteger:5], kOptionValue, nil]];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"10 meters", @"10 meters"), kOptionName, [NSNumber numberWithInteger:10], kOptionValue, nil]];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"50 meters", @"50 meters"), kOptionName, [NSNumber numberWithInteger:50], kOptionValue, nil]];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"100 meters", @"100 meters"), kOptionName, [NSNumber numberWithInteger:100], kOptionValue, nil]];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"500 meters", @"500 meters"), kOptionName, [NSNumber numberWithInteger:500], kOptionValue, nil]];
    [optionsAccuracy addObject:[NSDictionary dictionaryWithObjectsAndKeys:NSLocalizedString(@"1 kilometer", @"1 kilometer"), kOptionName, [NSNumber numberWithInteger:1000], kOptionValue, nil]];
}

@end
