//
//  SyncManager.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 03/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SyncManager.h"

@implementation SyncManager

@synthesize delegate, managedObjectContext, persistentStoreCoordinator, receivedData;

static SyncManager *sharedSyncManager = nil;


// Definició de la classe Singleton
+ (SyncManager *)sharedSyncManager
{
    if (sharedSyncManager == nil) {
        sharedSyncManager = [[super allocWithZone:NULL] init];
        sharedSyncManager.managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
        sharedSyncManager.persistentStoreCoordinator = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] persistentStoreCoordinator];
    }
    
    return sharedSyncManager;
}

+ (id)allocWithZone:(NSZone *)zone
{
    return [[self sharedSyncManager] retain];
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain
{
    return self;
}

- (NSUInteger)retainCount
{
    return NSUIntegerMax;
}

- (void)release
{
    
}

- (id)autorelease
{
    return self;
}

#pragma mark - Sync tasks register

- (void)registerSyncTaskType:(SyncTaskTypes)syncTaskType forManagedObject:(NSManagedObject *)managedObject
{
    NSManagedObjectID *objectID = [managedObject objectID];
    
    // Consultem si hi ha una altra operació pendent de l'objecte
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setPredicate:[NSPredicate predicateWithFormat:@"objectURI = %@", [[objectID URIRepresentation] absoluteString]]];
    [request setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    NSError *error = nil;
    NSArray *results = [managedObjectContext executeFetchRequest:request error:&error];
    [request release];
    if (error) {
        NSLog(@"ERROR fetching sync tasks");
    } else {
        SyncTask *syncTask = nil;
        if ([results count] > 0) {
            syncTask = [results lastObject];
            NSLog(@"Updating sync task for object: %@", [syncTask objectURI]);
            if (syncTaskType == SyncTaskDelete) {
                if ([[syncTask type] intValue] == SyncTaskCreate) {
                    [managedObjectContext deleteObject:syncTask];
                } else {
                    [syncTask setType:[NSNumber numberWithInt:syncTaskType]];
                }
            }
            [syncTask setUpdateDate:[NSDate date]];
            
        } else {
            NSLog(@"Creating sync task for object: %@", [[managedObject objectID] URIRepresentation]);
            syncTask = (SyncTask *)[NSEntityDescription insertNewObjectForEntityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext];
            [syncTask setState:[NSNumber numberWithInt:0]];
            [syncTask setObjectURI:[[objectID URIRepresentation] absoluteString]];
            [syncTask setType:[NSNumber numberWithInt:syncTaskType]];
            [syncTask setUpdateDate:[NSDate date]];
        }
        // Establim que hi ha dades per sincronitzar
        [[NSUserDefaults standardUserDefaults] setBool:YES forKey:@"dataToSync"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        if (![managedObjectContext save:&error]) {
            NSLog(@"ERROR SyncManager: couldn't save task");
        }
    }
    
}

- (void)clearAllSyncTasks
{
    
    // Es recuperen totes les tasques a sincronitzar pendents:
    //    - Si hi ha s'esborren.
    NSFetchRequest *syncTasksRequest = [[NSFetchRequest alloc] init];
    [syncTasksRequest setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    NSError *error = nil;
    NSArray *results = [[self managedObjectContext] executeFetchRequest:syncTasksRequest error:&error];
    [syncTasksRequest release];
    if (error) {
        NSLog(@"clearAllSyncTask - Error fetching all SyncTask: %@", [error localizedDescription]);
    } else {
        NSLog(@"clearAllSyncTasks: %i task(s) to clear", [results count]);
        if ([results count] > 0) {
            NSEnumerator *enumerator = [results objectEnumerator];
            SyncTask *aSyncTask = nil;
            while ((aSyncTask = (SyncTask *)[enumerator nextObject])) {
                [[self managedObjectContext] deleteObject:aSyncTask];
            }
            [[self managedObjectContext] save:&error];
            if (error) {
                NSLog(@"clearAllSyncTask - Error saving data: %@", [error localizedDescription]);
            }
        }
    }
}

- (void)createAllNewSyncTasks
{
    // Es recuperen totes les categories i es registra per sincronitzar-les
    NSFetchRequest *categories = [[NSFetchRequest alloc] init];
    [categories setEntity:[NSEntityDescription entityForName:@"Category" inManagedObjectContext:self.managedObjectContext]];
    NSError *error = nil;
    NSArray *cateogriesResults = [[self managedObjectContext] executeFetchRequest:categories error:&error];
    [categories release];
    if (error) {
        NSLog(@"createAllNewSyncTasks - Error fetching all Categories: %@", [error localizedDescription]);
    } else {
        NSLog(@"createAllNewSyncTasks: %i categories to register", [cateogriesResults count]);
        if ([cateogriesResults count] > 0) {
            NSEnumerator *enumerator = [cateogriesResults objectEnumerator];
            Category *aCategory = nil;
            while ((aCategory = (Category *)[enumerator nextObject])) {
                [self registerSyncTaskType:SyncTaskCreate forManagedObject:aCategory];
            }
            [[self managedObjectContext] save:&error];
            if (error) {
                NSLog(@"createAllSyncTasks - Error saving categories: %@", [error localizedDescription]);
            }
        }
    }
    
    // Es recuperen tots els recorreguts i es registren per la seua sincronització
    NSFetchRequest *tracks = [[NSFetchRequest alloc] init];
    [tracks setEntity:[NSEntityDescription entityForName:@"Track" inManagedObjectContext:self.managedObjectContext]];
    error = nil;
    NSArray *tracksResults = [[self managedObjectContext] executeFetchRequest:tracks error:&error];
    [tracks release];
    if (error) {
        NSLog(@"createAllNewSyncTasks - Error fetching all Categories: %@", [error localizedDescription]);
    } else {
        NSLog(@"createAllNewSyncTasks: %i tracks to register", [tracksResults count]);
        if ([tracksResults count] > 0) {
            NSEnumerator *enumerator = [tracksResults objectEnumerator];
            Track *aTrack = nil;
            while ((aTrack = (Track *)[enumerator nextObject])) {
                [self registerSyncTaskType:SyncTaskCreate forManagedObject:aTrack];
            }
            [[self managedObjectContext] save:&error];
            if (error) {
                NSLog(@"createAllSyncTasks - Error saving tracks: %@", [error localizedDescription]);
            }
        }
    }    
}

#pragma mark - Sync data

- (NSString *)dataToSendJSON
{
    // Recuperem les tasques a realitzar
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    [request setSortDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"updateDate" ascending:YES] autorelease]]];
    NSArray* syncTasks = [managedObjectContext executeFetchRequest:request error:nil];
    [request release];
    NSMutableDictionary *tasks = [[NSMutableDictionary alloc] init];
    NSMutableDictionary *tasksData = [[NSMutableDictionary alloc] init];
    
    // Per cada tasca:
    //    - Si es necessita recuperem les dades i les preparem per ser enviades a tasksData
    //    - S'afegeix l'operació a realitzar
    for (NSInteger i = 0; i < [syncTasks count]; i++) {
        SyncTask *aTask = (SyncTask *)[syncTasks objectAtIndex:i];
        if ([[aTask type] intValue] != SyncTaskDelete ) {
            NSLog(@"Preparing %@", [aTask objectURI]);
            NSManagedObjectID *objectID = [persistentStoreCoordinator managedObjectIDForURIRepresentation:[NSURL URLWithString:[aTask objectURI]]];
            if ([objectID entity] == [NSEntityDescription entityForName:@"Category" inManagedObjectContext:managedObjectContext]) {
                Category *aCategory = (Category *)[managedObjectContext objectWithID:objectID];
                [tasksData setObject:[aCategory encodeJSON] forKey:[aTask objectURI]];
            } else if ([objectID entity] == [NSEntityDescription entityForName:@"Track" inManagedObjectContext:managedObjectContext]) {
                Track *aTrack = (Track*)[managedObjectContext objectWithID:objectID];
                [tasksData setObject:[aTrack encodeJSON] forKey:[aTask objectURI]];
            }
        }
        [tasks setObject:[aTask encodeJSON] forKey:[NSString stringWithFormat:@"%i", i]];
    }
    
    // Es preparen les credencials
    NSDictionary *credentials = [[NSDictionary alloc] initWithObjectsAndKeys:
                                 [[NSUserDefaults standardUserDefaults] valueForKey:@"Nickname"], @"nickname",
                                 [[NSUserDefaults standardUserDefaults] valueForKey:@"Password"], @"key", nil];
    
    // Les tasques i dades dels objectes s'afegeixen a un mateix objecte que els conté
    NSDictionary *data = [[NSDictionary alloc] initWithObjectsAndKeys:
                          [credentials JSONRepresentation], @"credentials",
                          [tasks JSONRepresentation], @"tasks",
                          [tasksData JSONRepresentation], @"tasksData", nil];
    [credentials release];
    NSLog(@"Data sent: %@",[data JSONRepresentation]);
    [tasks release];
    [tasksData release];
    NSString *stringResult = [data JSONRepresentation];
    [data release];
    return stringResult;
}

#pragma mark - Sync actions

- (void)syncWithDelegate:(id<SyncManagerProtocol>)aDelegate
{
    [self setDelegate:aDelegate];
    [self syncRequest];
}

- (void)syncRequest
{
    [(UIApplication *)[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
    NSMutableURLRequest *syncRequest = [[[NSMutableURLRequest alloc] initWithURL:[NSURL URLWithString:kWebAppURL]] autorelease];
    NSString *dataToSend = @"data=";
    dataToSend = [dataToSend stringByAppendingString:[self dataToSendJSON]];
    NSString *dataLength = [NSString stringWithFormat:@"%i", [dataToSend lengthOfBytesUsingEncoding:NSUTF8StringEncoding]];
    [syncRequest addValue:dataLength forHTTPHeaderField:@"Content-Length"];
    [syncRequest setHTTPMethod:@"POST"];
    [syncRequest addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [syncRequest setHTTPBody:[dataToSend dataUsingEncoding:NSUTF8StringEncoding]];
    NSURLConnection *theConnection = [[NSURLConnection alloc] initWithRequest:syncRequest delegate:self];
    if (!theConnection) {
        UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Connection can't be established" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alertView show];
        [alertView release];
    } else {
        receivedData = [[NSMutableData alloc] init];
    }
}

- (void)syncResponse:(NSDictionary *)result
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    NSArray *syncTasks = [managedObjectContext executeFetchRequest:request error:nil];
    [request release];
//    if ([[result objectForKey:@"resultCode"] intValue] == 1) {
//        NSEnumerator *enumerator = [[result objectForKey:@"conflicts"] objectEnumerator];
//    }
    // Comprovem i actualitzem les dades rebudes de la Web
    NSDictionary *tasksReceived = [result objectForKey:@"tasks"];
    if (tasksReceived) {
        NSDictionary *dataReceived = [result objectForKey:@"data"];
        for (NSInteger i = 0; i < [tasksReceived count]; i++) {
            NSDictionary *aTaskToDo = [tasksReceived objectForKey:[NSString stringWithFormat:@"%i", i]];
            if ([[aTaskToDo objectForKey:@"state"] intValue] == 0 || [[aTaskToDo objectForKey:@"state"] intValue] == 3) {
                NSLog(@"Processing task %i: %@", i, [aTaskToDo objectForKey:@"objectURI"]);
                
                // Recuperem l'objecte per fer les modificiacions
                NSManagedObjectID *objectID = [persistentStoreCoordinator managedObjectIDForURIRepresentation:[NSURL URLWithString:[aTaskToDo objectForKey:@"objectURI"]]];
                NSError *error = nil;
                NSManagedObject *anObject = [managedObjectContext existingObjectWithID:objectID error:&error];
                if (error) {
                    NSLog(@"Error retrieving %@: %@", [objectID URIRepresentation], [error localizedDescription]);
                }
                
                // Apliquem l'operació sobre l'objecte
                if ([[aTaskToDo objectForKey:@"type"] intValue] == SyncTaskDelete) {
                    NSLog(@"Task %i: deleting %@", i, [[anObject objectID] URIRepresentation]);
                    [managedObjectContext deleteObject:anObject];
                } else {
                    NSDictionary *objectNewData = [dataReceived objectForKey:[aTaskToDo objectForKey:@"objectURI"]];
                    // Apliquem nous valors
                    NSLog(@"Task %i: updating %@", i, [[anObject objectID] URIRepresentation]);
                    NSLog(@"Change %@ to %@", [anObject valueForKey:@"name"], [objectNewData objectForKey:@"name"]);
                    [anObject setValue:[objectNewData objectForKey:@"name"] forKey:@"name"];
                    if ([[aTaskToDo objectForKey:@"objectURI"] rangeOfString:@"Track"].location != NSNotFound) {
                        [anObject setValue:[objectNewData objectForKey:@"ppm"] forKey:@"ppm"];
                        [anObject setValue:[objectNewData objectForKey:@"note"] forKey:@"note"];
                        Category *newCategory = (Category*)[managedObjectContext existingObjectWithID:[persistentStoreCoordinator managedObjectIDForURIRepresentation:[NSURL URLWithString:[objectNewData objectForKey:@"category"]]] error:nil];
                        [anObject setValue:newCategory forKey:@"category"];
                    }
                }
            } else {
                NSFetchRequest *taskRequest = [[NSFetchRequest alloc] init];
                [taskRequest setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
                [taskRequest setPredicate:[NSPredicate predicateWithFormat:@"objectURI=%@", [aTaskToDo objectForKey:@"objectURI"]]];
                NSArray *results = [managedObjectContext executeFetchRequest:taskRequest error:nil];
                [[results lastObject] setValue:[aTaskToDo objectForKey:@"state"] forKey:@"state"];
                [taskRequest release];
            }
        }
        NSError *error = nil;
        [managedObjectContext save:&error];
        if (error) {
            NSLog(@"Error saving updated data: %@", [error localizedDescription]);
        }
    }
    
    NSEnumerator *tasksEnumerator = [syncTasks objectEnumerator];
    SyncTask *aSyncTask = nil;
    while ((aSyncTask = [tasksEnumerator nextObject])) {
        if ([[aSyncTask state] intValue] != 1) {
            NSLog(@"SyncTask to clean: %@", [[aSyncTask objectID] URIRepresentation]);
            [managedObjectContext deleteObject:aSyncTask];
        }
    }
    NSError *error = nil;
    if (![managedObjectContext save:&error]) {
        NSLog(@"ERROR: Couldn't delete SyncTask: %@", [error localizedDescription]);
    }
    [[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"dataToSync"];
    
    // Comprovem si hi ha conflictes pendent de resoldre
    NSFetchRequest *conflictsRequest = [[NSFetchRequest alloc] init];
    [conflictsRequest setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    [conflictsRequest setPredicate:[NSPredicate predicateWithFormat:@"state=%i", 1]];
    error = nil;
    NSArray *conflictTasks = [self.managedObjectContext executeFetchRequest:conflictsRequest error:&error];
    BOOL thereAreConflicts = [conflictTasks count] > 0 ? YES : NO;
    if ([delegate respondsToSelector:@selector(syncManagerDidEndSyncWithConflicts:)]) {
        [delegate syncManagerDidEndSyncWithConflicts:thereAreConflicts];
    }
    delegate = nil;
    [conflictsRequest release];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    [receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    [receivedData appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    NSString *jsonData = [[NSString alloc] initWithData:receivedData encoding:NSUTF8StringEncoding];
    NSLog(@"Raw string: %@", jsonData);
    NSLog(@"JSON: %@", [[jsonData JSONValue] objectForKey:@"resultCode"]);
    if ([[jsonData JSONValue] objectForKey:@"resultCode"]) {
        if ([[[jsonData JSONValue] objectForKey:@"resultCode"] intValue] >= 0) {
            [self syncResponse:[jsonData JSONValue]];
        } else {
            if ([[jsonData JSONValue] objectForKey:@"log"]) {
                UIAlertView *alertView = [[UIAlertView alloc]
                                          initWithTitle:NSLocalizedString(@"Error", @"Error") 
                                          message:[[jsonData JSONValue] objectForKey:@"log"] 
                                          delegate:nil 
                                          cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                          otherButtonTitles:nil];
                [alertView show];
                [alertView release];
            }
        }
    } else {
        UIAlertView *alertView = [[UIAlertView alloc]
                                  initWithTitle:NSLocalizedString(@"Error", @"Error") 
                                  message:NSLocalizedString(@"Internal server error", @"Internal server error") 
                                  delegate:nil 
                                  cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                  otherButtonTitles:nil];
        [alertView show];
        [alertView release];
    }
    [jsonData release];
    [connection release];
    [(UIApplication *)[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    NSLog(@"Connection failed: %@", [error localizedDescription]);
    UIAlertView *alertView = [[UIAlertView alloc]
                              initWithTitle:NSLocalizedString(@"Connection Error", @"Alert view - Title")
                              message:[error localizedDescription] 
                              delegate:nil
                              cancelButtonTitle:NSLocalizedString(@"OK", @"OK")
                              otherButtonTitles:nil];
    [alertView show];
    [alertView release];
    [(UIApplication *)[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
}

@end
