//
//  Track.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 03/05/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "Track.h"
#import "Category.h"
#import "Place.h"
#import "PlaceMark.h"
#import "SimilarTracks.h"


@implementation Track
@dynamic maxAltitude;
@dynamic maxSpeed;
@dynamic updated;
@dynamic note;
@dynamic ppm;
@dynamic time;
@dynamic distance;
@dynamic date;
@dynamic dataKML;
@dynamic name;
@dynamic minAltitude;
@dynamic placemarks;
@dynamic category;
@dynamic similarTracks;
@dynamic places;

- (void)addPlacemarksObject:(Placemark *)value {    
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"placemarks"] addObject:value];
    [self didChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
    [changedObjects release];
}

- (void)removePlacemarksObject:(Placemark *)value {
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"placemarks"] removeObject:value];
    [self didChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
    [changedObjects release];
}

- (void)addPlacemarks:(NSSet *)value {    
    [self willChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"placemarks"] unionSet:value];
    [self didChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
}

- (void)removePlacemarks:(NSSet *)value {
    [self willChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"placemarks"] minusSet:value];
    [self didChangeValueForKey:@"placemarks" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
}





- (void)addPlacesObject:(Place *)value {    
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"places" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"places"] addObject:value];
    [self didChangeValueForKey:@"places" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
    [changedObjects release];
}

- (void)removePlacesObject:(Place *)value {
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"places" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"places"] removeObject:value];
    [self didChangeValueForKey:@"places" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
    [changedObjects release];
}

- (void)addPlaces:(NSSet *)value {    
    [self willChangeValueForKey:@"places" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"places"] unionSet:value];
    [self didChangeValueForKey:@"places" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
}

- (void)removePlaces:(NSSet *)value {
    [self willChangeValueForKey:@"places" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"places"] minusSet:value];
    [self didChangeValueForKey:@"places" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
}

- (float)distanceInKm
{
    return [[self distance] floatValue] / 1000;
}

- (float)speedInKmH
{
    return ([[self distance] floatValue]/ 1000) / ([[self time] floatValue] / 3600);
}

- (float)maxSpeedInKmH
{
    return ([[self maxSpeed] floatValue] * 3.6);
}

- (float)pace
{
    return [[self distance] floatValue] == 0.0 ? 0.0 : ([[self time] floatValue] / 60) / ([[self distance] floatValue] / 1000);
}

- (NSString *)paceString
{
    NSInteger min = (NSInteger)[self pace];
    NSInteger sec = (NSInteger)(([self pace] - min) * 60);
    return [NSString stringWithFormat:@"%i:%02i", min, sec];
}

- (void)setUpdatedTimestamp
{
    [self setUpdated:[NSDate date]];
}

- (NSString*)formatTime
{
    NSTimeInterval theInterval = [[self time] floatValue];
    int hours = 0;
    int minutes = 0;
    int seconds = 0;
    if (theInterval >= 60*60) {
        hours = round(theInterval / 60*60);
    }
    if (theInterval - hours*60*60 >= 60) {
        minutes = floor((theInterval - hours * 60 * 60) / 60);
    }
    seconds = floor(theInterval - hours*60*60 - minutes*60);
    NSString *result;
    if (hours == 0) {
        result = [NSString stringWithFormat:@"%02i:%02i", minutes, seconds];
    } else {
        result = [NSString stringWithFormat:@"%i:%02i:%02i", hours, minutes, seconds];
    }        
    return result;
}

- (void)writeKMLData
{
    NSArray *sortedPlaces = [[self places] sortedArrayUsingDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"time" ascending:YES] autorelease]]];
    NSLog(@"Start writeKMLData");
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSURL *KMLDirectory = [NSURL fileURLWithPath:[[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:@"KMLFiles"]];
    NSDateComponents *dateComponents = [[NSCalendar currentCalendar] components:(NSYearCalendarUnit | NSMonthCalendarUnit | NSDayCalendarUnit | NSHourCalendarUnit | NSMinuteCalendarUnit ) fromDate:[self date]];
    NSString *KMLFileName = [NSString stringWithFormat:@"%i%02i%02i%02i%02i - %@.kml", [dateComponents year], [dateComponents month], [dateComponents day], [dateComponents hour], [dateComponents minute], [self name]];
    NSLog(@"dataKML: %@", [self dataKML]);
    if ([self dataKML] == nil || [[self dataKML] length] == 0) {
        [self setDataKML:KMLFileName];
        NSLog(@"KML File: %@", [[KMLDirectory path] stringByAppendingPathComponent:[self dataKML]]);
        [fileManager createFileAtPath:[[KMLDirectory path] stringByAppendingPathComponent:[self dataKML]] contents:nil attributes:nil];
    }
    if ([self dataKML] != KMLFileName) {
        [fileManager removeItemAtPath:[[KMLDirectory path] stringByAppendingPathComponent:[self dataKML]] error:nil];
        [self setDataKML:KMLFileName];
        [fileManager createFileAtPath:[[KMLDirectory path] stringByAppendingPathComponent:[self dataKML]] contents:nil attributes:nil];
    }
    
    NSString *fileContent = @"";
    // Inici de l'arxiu KML
    fileContent = [fileContent stringByAppendingString:@"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<kml xmlns=\"http://www.opengis.net/kml/2.2\"\nxmlns:gx=\"http://www.google.com/kml/ext/2.2\">\n<Document>\n"];
    // Estil de la línia
    fileContent = [fileContent stringByAppendingString:
                   @"<Style id=\"line\">\n\t<LineStyle>\n\t\t<color>7fff00ff</color>\n\t\t<width>4</width>\n\t</LineStyle>\n\t<PolyStyle>\n\t\t<color>7f00ff00</color>\n\t</PolyStyle>\n</Style>\n"];
    // Marca d'inici
    fileContent = [fileContent stringByAppendingFormat:
                   @"<Placemark>\n<name>%@</name>\n\t<Point>\n\t\t<coordinates>%f,%f</coordinates>\n\t</Point>\n</Placemark>\n",
                   NSLocalizedString(@"Start", @"Start"),
                   [[[sortedPlaces objectAtIndex:0] longitude] floatValue],
                   [[[sortedPlaces objectAtIndex:0] latitude] floatValue]];
    // Marca de fi
    fileContent = [fileContent stringByAppendingString:
                   [NSString stringWithFormat:@"<Placemark>\n<name>%@</name>\n\t<description>%@</description>\n\t<Point>\n\t\t<coordinates>%f,%f</coordinates>\n\t</Point>\n</Placemark>\n",
                    NSLocalizedString(@"End", @"End"),
                    [NSString localizedStringWithFormat:@"%@ - %.2fkm - %@m/km", [self formatTime], [self distanceInKm], [self paceString]],
                    [[[sortedPlaces lastObject] longitude] floatValue],
                    [[[sortedPlaces lastObject] latitude] floatValue]]];
    
    // Si hi ha placemarks es situen
    if ([[self placemarks] count] > 1) {
        NSArray *sortedPlacemarks = [[self placemarks] sortedArrayUsingDescriptors:[NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"startTime" ascending:YES] autorelease]]];
        for (NSInteger i = 0; i < [sortedPlacemarks count] - 1; i++) {
            Placemark *aPlacemark = [sortedPlacemarks objectAtIndex:i];
            fileContent = [fileContent stringByAppendingString:
                           [NSString stringWithFormat:@"<Placemark>\n<name>%@</name>\n\t<description>%@</description>\n\t<Point>\n\t\t<coordinates>%f,%f</coordinates>\n\t</Point>\n</Placemark>\n",
                            [NSString localizedStringWithFormat:@"%i - %@", i+1, [aPlacemark name]],
                            [NSString localizedStringWithFormat:@"%@ - %.2fm - %@m/km", [aPlacemark formatTime], [[aPlacemark distance] floatValue], [aPlacemark paceString]],
                            [[[aPlacemark place] longitude] floatValue],
                            [[[aPlacemark place] latitude] floatValue]]];

        }
    }
    
    // Línia
    fileContent = [fileContent stringByAppendingFormat:@"<Placemark>\n\t<name></name>\n\t<visibility>1</visibility>\n\t<styleUrl>#line</styleUrl>\n\t<LineString>\n\t<coordinates>\n"];
    for (NSInteger i = 0; i < [sortedPlaces count]; i++) {
        Place *aPlace = [sortedPlaces objectAtIndex:i];
        fileContent = [fileContent stringByAppendingFormat:@"\t\t%f,%f\n", [[aPlace longitude]floatValue], [[aPlace latitude] floatValue]];
        
    }
    fileContent = [fileContent stringByAppendingFormat:@"\t</coordinates>\n\t</LineString>\n</Placemark>\n"];
    
    // Fi de l'arxiu KML
    fileContent = [fileContent stringByAppendingString:@"</Document>\n</kml>"];
    NSLog(@"Content to write: %@", fileContent);
    NSError *error = nil;
    if (![fileContent writeToFile:[[KMLDirectory path] stringByAppendingPathComponent:[self dataKML]] atomically:YES encoding:NSUTF8StringEncoding error:&error]) {
        NSLog(@"Error: %@", [error localizedFailureReason]);
    }
     NSLog(@"Writing to file");
    NSLog(@"End writeKMLData");
}

- (NSDictionary *)toDictionary
{
    NSMutableDictionary* objectDictionary = [[NSMutableDictionary alloc] init];
    [objectDictionary setObject:[[[self objectID] URIRepresentation] absoluteString] forKey:@"id"];
    [objectDictionary setObject:[self name] forKey:@"name"];
    [objectDictionary setObject:[[self date] description] forKey:@"date"];
    [objectDictionary setObject:[self time] forKey:@"time"];
    [objectDictionary setObject:[self minAltitude] forKey:@"minAltitude"];
    [objectDictionary setObject:[self maxAltitude] forKey:@"maxAltitude"];
    [objectDictionary setObject:[self maxSpeed] forKey:@"maxSpeed"];
    [objectDictionary setObject:[self distance] forKey:@"distance"];
    [objectDictionary setObject:[self minAltitude] forKey:@"minAltitude"];
    [objectDictionary setObject:[self ppm] forKey:@"ppm"];
    [objectDictionary setObject:([self note] == nil) ? @"" : [self note] forKey:@"note"];
    [objectDictionary setObject:[[[[self category] objectID] URIRepresentation] absoluteString] forKey:@"category"];
    [objectDictionary setObject:[[self updated] description] forKey:@"updated"];
    
    // Si hi ha marques es preparen
    if ([[self placemarks] count] > 0) {
        NSMutableDictionary *placemarksDictionary = [[NSMutableDictionary alloc] init];
        NSEnumerator *enumerator = [[self placemarks] objectEnumerator];
        Placemark *aPlacemark = nil;
        while ((aPlacemark = [enumerator nextObject])) {
            [placemarksDictionary setObject:[aPlacemark toDictionary] forKey:[[[aPlacemark objectID] URIRepresentation] path]];
        }
        [objectDictionary setObject:placemarksDictionary forKey:@"placemarks"];
    }
    
    // Dades del fitxer KML
    NSString *path = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] applicationDocumentsDirectory];
    path = [path stringByAppendingPathComponent:[NSString stringWithFormat:@"KMLFiles/%@",[self dataKML]]];
    NSError *error = nil;
    NSString *fileContents = [[NSString alloc] initWithContentsOfFile:path encoding:NSUTF8StringEncoding error:&error];
    if (error) {
        NSLog(@"ERROR: %@\nFile: %@", [error localizedDescription], path);
    }
    [objectDictionary setObject:fileContents forKey:@"dataKML"];
    return objectDictionary;
}

- (NSString *)encodeJSON
{
    return [[self toDictionary] JSONRepresentation];
}
@end
