//
//  SyncManager.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 03/05/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "TrackLocationManager.h"

@implementation TrackLocationManager

@synthesize delegate, locationManager;

static TrackLocationManager *sharedTrackLocationManager = nil;


// Definició de la classe Singleton
+ (TrackLocationManager *)sharedTrackLocationManager
{
    if (sharedTrackLocationManager == nil) {
        sharedTrackLocationManager = [[super allocWithZone:NULL] init];
    }
    
    return sharedTrackLocationManager;
}

+ (id)allocWithZone:(NSZone *)zone
{
    return [[self sharedTrackLocationManager] retain];
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain
{
    return self;
}

- (NSUInteger)retainCount
{
    return NSUIntegerMax;
}

- (void)release
{
    
}

- (id)autorelease
{
    return self;
}

#pragma mark - Getters

// Retorna el valor registerPlacemarks de les preferències d'usuari
- (BOOL)registerPlacemarks
{
    return [[NSUserDefaults standardUserDefaults] boolForKey:@"Placemarks"];
}

// Retorna el valor distanceBetweenPlacemarks de les preferències d'usuari
- (NSInteger)distanceBetweenPlacemarks
{
    return [[NSUserDefaults standardUserDefaults] integerForKey:@"MarksDistance"];
}

// Retorna el valor distanceFilter de les preferències d'usuari
- (NSInteger)distanceFilter
{
    return [[NSUserDefaults standardUserDefaults] integerForKey:@"Accuracy"];
}

#pragma mark - Location delegate

/*
 * Mètode delegat quan locationManager retorna una nova posició
 */
- (void)locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation
{
    if (delegate && [delegate respondsToSelector:@selector(trackLocationManager:didUpdateToLocation:fromLocation:)]) {
        NSLog(@"sharedLocationManager: passem les dades al delegat");
        [[sharedTrackLocationManager delegate] trackLocationManager:sharedTrackLocationManager didUpdateToLocation:newLocation fromLocation:oldLocation];
    }
}

/*
 * Mètode del delegat quan locationManager retorna un error.
 */
- (void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error
{
    if (error.code == kCLErrorLocationUnknown) {
        NSLog(@"No s'ha pogut obtindre la localització");
    } else if (error.code == kCLErrorNetwork) {
        NSLog(@"La xarxa per obtindre la localització no està disponible");
    } else if (error.code == kCLErrorDenied) {
        NSLog(@"L'usuari no vol que el localitzen");
        UIAlertView *alertLocationDisabled = [[UIAlertView alloc]
                                              initWithTitle:@"Error"
                                              message:NSLocalizedString(@"Corre hasn't authorisation for retrieve localization", @"Error - user authorization") 
                                              delegate:self
                                              cancelButtonTitle:NSLocalizedString(@"OK", @"OK") 
                                              otherButtonTitles:nil];
        [alertLocationDisabled show];
        [alertLocationDisabled release];
    }
    [[sharedTrackLocationManager locationManager] stopUpdatingLocation];
}


#pragma mark - Location tasks

- (void)wakeUpLocationManager
{
    if (sharedTrackLocationManager.locationManager == nil) {
        sharedTrackLocationManager.locationManager = [[[CLLocationManager alloc] init] autorelease];
        sharedTrackLocationManager.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        sharedTrackLocationManager.locationManager.distanceFilter = [sharedTrackLocationManager distanceFilter];
        sharedTrackLocationManager.locationManager.delegate = self;
    }
    [[sharedTrackLocationManager locationManager] startUpdatingLocation];
}

- (void)shutDownLocationManager
{
    if (sharedTrackLocationManager.locationManager != nil) {
        [[sharedTrackLocationManager locationManager] stopUpdatingLocation];
        [sharedTrackLocationManager setLocationManager:nil];
    }
}

@end