//
//  TracksTableViewController.m
//  Corre
//
//  Created by Pedro Nofuentes Azcárate on 27/04/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "TracksTableViewController.h"


@implementation TracksTableViewController

@synthesize managedObjectContext;
@synthesize tracks;
@synthesize conflictsTableViewController;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [conflictsTableViewController release];
    [managedObjectContext release];
    [tracks release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - Getters and setters

- (void)setDetailTrackViewController:(DetailTrackViewController *)aDetailTrackViewController
{
    self.detailTrackViewController = aDetailTrackViewController;
}

- (DetailTrackViewController *)detailTrackViewController
{
    if (detailTrackViewController == nil) {
        detailTrackViewController = [[DetailTrackViewController alloc] initWithNibName:@"DetailTrackView" bundle:nil];
        detailTrackViewController.hidesBottomBarWhenPushed = YES;
    }
    
    return detailTrackViewController;
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    self.managedObjectContext = [(CorreAppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    self.navigationItem.rightBarButtonItem = [self editButtonItem];
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(showConflictsAndSync)];    
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    // Carreguem les dades dels recorreguts
    [self loadTracks];
    
    // Si no estan definit usuari i contrasenya deshabilitem el botó de sincronització. En cas contrar l'habilitem.
    if ([[[NSUserDefaults standardUserDefaults] stringForKey:@"Nickname"] isEqualToString:@""] || [[[NSUserDefaults standardUserDefaults] stringForKey:@"Password"] isEqualToString:@""]) {
        [[[self navigationItem] leftBarButtonItem] setEnabled:NO];
    } else {
        [[[self navigationItem] leftBarButtonItem] setEnabled:YES];
    }
}

#pragma mark - Data methods

- (void)loadTracks
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSArray *sortDescriptors = [NSArray arrayWithObject:[[[NSSortDescriptor alloc] initWithKey:@"date" ascending:NO] autorelease]];
    [request setSortDescriptors:sortDescriptors];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Track" inManagedObjectContext:self.managedObjectContext];
    [request setEntity:entity];
    NSError *error = nil;
    tracks = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    NSLog(@"Number of Tracks: %i", [tracks count]);
    [request release];
    if (error) {
        NSLog(@"!Error retrieving tracks: %@", [error description]);
    }
    [[self tableView] reloadData];
}

#pragma mark - Table delegate methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [tracks count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *MyIdentifier = @"TracksCellIdentifier";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:MyIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:MyIdentifier] autorelease];
        [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
    }
    
    Track * track = (Track *) [tracks objectAtIndex:indexPath.row];
    [[cell textLabel] setText:[track name]];
    [[cell detailTextLabel] setText:[NSString localizedStringWithFormat:@"%.2f km - %@ - %@: %@m/km", [track distanceInKm], [track formatTime], NSLocalizedString(@"Pace",@"Pace"),[track paceString]]];
    track = nil;
                                                                                                                                                                                                             
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    DetailTrackViewController *detailTrack = [[DetailTrackViewController alloc] initWithNibName:@"DetailTrackView" bundle:nil];
    detailTrack.hidesBottomBarWhenPushed = YES;
    [detailTrack setTheTrack:(Track *)[tracks objectAtIndex:indexPath.row]];
    [[self navigationController] pushViewController:detailTrack animated:YES];
    [detailTrack release];
}

#pragma mark - Table edit methods

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
    [super setEditing:editing animated:animated];
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [(SyncManager *)[SyncManager sharedSyncManager] registerSyncTaskType:SyncTaskDelete forManagedObject:[tracks objectAtIndex:indexPath.row]];
        [managedObjectContext deleteObject:[tracks objectAtIndex:indexPath.row]];
        NSError *error = nil;
        if ([managedObjectContext save:&error]) {
            [tracks removeObjectAtIndex:indexPath.row];
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObjects:indexPath, nil] withRowAnimation:UITableViewRowAnimationRight];
        } else {
            NSLog(@"Error deleting: %@", [error description]);
        }
    }
}

#pragma mark - Sync method

- (void)syncronizeTracks
{
    [(SyncManager *)[SyncManager sharedSyncManager] syncWithDelegate:self];
}

- (void)showConflictsAndSync
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    [request setPredicate:[NSPredicate predicateWithFormat:@"state!=%i",0]];
    NSError *error = nil;
    NSArray *results = [self.managedObjectContext executeFetchRequest:request error:&error];
    if (error) {
        NSLog(@"Error fetching SyncTask: %@", [error localizedDescription]);
    } else if ([results count] == 0){
        [self syncronizeTracks];
    } else {
        if (conflictsTableViewController == nil) {
            conflictsTableViewController = [[ConflictsTableViewController alloc] initWithNibName:@"ConflictsTableView" bundle:nil delegate:self];
        }
        [self presentModalViewController:conflictsTableViewController animated:YES];
    }
}

- (void)checkConflicts
{
    NSLog(@"checkConflicts");
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:[NSEntityDescription entityForName:@"SyncTask" inManagedObjectContext:self.managedObjectContext]];
    [request setPredicate:[NSPredicate predicateWithFormat:@"state==%i",1]];
    NSError *error = nil;
    NSArray *results = [self.managedObjectContext executeFetchRequest:request error:&error];
    [request release];
    NSLog(@"Conflicts state==1: %i", [results count]);
    if (error) {
        NSLog(@"Error fetching SyncTask: %@", [error localizedDescription]);
    } else if ([results count] == 0){
        UIAlertView *alertView = [[UIAlertView alloc] 
                                  initWithTitle:NSLocalizedString(@"Syncronization", @"Alert view title") 
                                  message:NSLocalizedString(@"No unresolved conflicts. Do you want to sync now?", @"Alert view message") 
                                  delegate:self 
                                  cancelButtonTitle:NSLocalizedString(@"No", @"No") 
                                  otherButtonTitles:NSLocalizedString(@"Yes", @"Yes"), nil];
        [alertView show];
    }
}

- (void)syncManagerDidEndSyncWithConflicts:(BOOL)thereAreConflicts
{
    if (thereAreConflicts) {
        if (conflictsTableViewController == nil) {
            conflictsTableViewController = [[ConflictsTableViewController alloc] initWithNibName:@"ConflictsTableView" bundle:nil delegate:self];
        }
        [self presentModalViewController:self.conflictsTableViewController animated:YES];
    }
    [self loadTracks];
}

- (void)conflictsTableViewControllerDidEndEditing
{
    NSLog(@"conflictsTableViewControllerDidEndEditing");
    [self checkConflicts];
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    NSString *buttonClicked = [alertView buttonTitleAtIndex:buttonIndex];
    if (buttonClicked == NSLocalizedString(@"Yes", @"Yes")) {
        [self syncronizeTracks];
    }
}

@end
