# -*- coding:utf-8 -*-
from datetime import datetime, date
from django.utils import simplejson as json
from django.contrib.auth.models import User
from core.models import Category, Track, Placemark, SyncTask
from core.utils import dateFromNSString

def processTaskFromUser(task, user, taskData):
  if task['type'] == 0:
    if taskData['id'].find('Category') >= 0:
      message = createCategory(taskData, user)
    elif taskData['id'].find('Track') >= 0:
      message = createTrack(taskData, user)
    else:
      message = { u'Error': u'Unknown object'}
  elif task['type'] == 1:
    message = modifyTrack(task, taskData, user)
  elif task['type'] ==2:
    if task['objectURI'].find('Category') >= 0:
      message = deleteCategory(task, user)
    elif task['objectURI'].find('Track') >= 0:
      message = deleteTrack(task, user)
    else:
      message = {u'Error' : u'Unknown object'}
  else:
    message = u'Unknown task'
  return message

def deleteObject(objectURI):
  return u'Esborrem objecte'


def createCategory(categoryData, user):
  result = {}
  category = Category(objectURI=categoryData['id'], name=categoryData['name'], user=user, updated=dateFromNSString(categoryData['updated']))
  category.save()
  if category.id:
    result = {u'OK' : u'Category created: %s' % category.objectURI}
  else:
    result = {u'Error' : u'Couldn\'t create Category: %s' % categoryData['id']}
  return result


def createTrack(trackData, user):
  track = Track(objectURI=trackData['id'], user=user)
  track.name = trackData['name']
  trackCategory = Category.objects.get(objectURI=trackData['category'], user=user)
  track.category = trackCategory
  track.date = dateFromNSString(trackData['date'])
  track.time = float(trackData['time'])
  track.distance = float(trackData['distance'])
  track.minAltitude = float(trackData['minAltitude'])
  track.maxAltitude = float(trackData['maxAltitude'])
  track.maxSpeed = float(trackData['maxSpeed'])
  track.ppm = int(trackData['ppm'])
  track.note = trackData['note']
  track.dataKML = trackData['dataKML']
  track.updated = dateFromNSString(trackData['updated'])
  track.save()
  if track.id:
    trackResult = {u'OK' : u'Track created: %s' % track.objectURI }
  else:
    trackResult = {u'Error' : u'Couldn\'t create Track: %s' % trackData['id']}
  placemarksData = u''
  if trackData.has_key(u'placemarks'):
    placemarksResults = {}
    i = 0
    for key in trackData['placemarks'].keys():
        placemarksResults[i] = createPlacemark(trackData['placemarks'][key], track, user)
        i += 1
  result = {'track': trackResult, 'placemarks': placemarksResults}
  return result

def createPlacemark(placemarkData, track, user):
  result = {}
  placemark = Placemark(objectURI=placemarkData['id'], track=track, user=user)
  placemark.name = placemarkData['name']
  placemark.startTime = dateFromNSString(placemarkData['startTime'])
  placemark.time = float(placemarkData['time'])
  placemark.distance = float(placemarkData['distance'])
  placemark.minAltitude = float(placemarkData['minAltitude'])
  placemark.maxAltitude = float(placemarkData['maxAltitude'])
  placemark.maxSpeed = float(placemarkData['maxSpeed'])
  placemark.save()
  if placemark.id:
    result = {u'OK' : u'Placemark created: %s' % placemark.objectURI }
  else:
    result = {u'Error' : u'Couldn\'t create Placemark: %s' % placemarkData['id']}

def modifyTrack(task, trackData, user):
  result = {}
  writeData = False
  serverSyncTask = None
  if task['state'] == 0:
    try:
      serverSyncTask = SyncTask.objects.get(objectURI=trackData['id'], user=user)
    except SyncTask.DoesNotExist:
      writeData = True
    if serverSyncTask:
      serverSyncTask.stateTask = 1
      serverSyncTask.save()
      result = {u'OK' : u'Conflict detected: URI %s set to state %d' % (trackData['id'], serverSyncTask.stateTask)}
  elif task['state'] == 2:
    writeData = True
    serverSyncTask = SyncTask.objects.get(objectURI=trackData['id'], user=user)
    serverSyncTask.delete()
  elif task['state'] == 3:
    serverSyncTask = SyncTask.objects.get(objectURI=trackData['id'], user=user)
    serverSyncTask.stateTask = 3
    serverSyncTask.save()
    result = {u'OK' : u'SyncTask resolved URI %s state %d' % (serverSyncTask.objectURI, serverSyncTask.stateTask)}
  if writeData:
    track = Track.objects.get(objectURI=trackData['id'], user=user)
    track.name = trackData['name']
    trackCategory = Category.objects.get(objectURI=trackData['category'])
    track.category = trackCategory
    track.ppm = int(trackData['ppm'])
    track.note = trackData['note']
    track.dataKML = trackData['dataKML']
    track.save()
    if track.id:
      result = {u'OK' : u'Track modified: %s' % track.objectURI }
    else:
      result = {u'Error' : u'Couldn\'t modify Track: %s' % trackData['id']}

  return result

def deleteCategory(task, user):
  try:
    category = Category.objects.get(objectURI=task['objectURI'], user=user)
    category.delete()
    result = {u'OK' : u'Category deleted: %s' % task['objectURI']}
  except Category.DoesNotExist:
    result = {u'Error' : u'Category don\'t found: %s' % task['objectURI']}
  try:
    serverSyncTask = SyncTask.objects.get(objectURI=task['objectURI'], user=user)
    serverSyncTask.delete()
  except SyncTask.DoesNotExist:
    serverSyncTask = None
  return result

def deleteTrack(task, user):
  try:
    track = Track.objects.get(objectURI=task['objectURI'], user=user)
    placemarks = Placemark.objects.filter(track=track, user=user)
    placemarksResult = {}
    i = 0
    for placemark in placemarks:
      placemarksResult[i] = {u'OK' : u'Placemark deleted: %s' % placemark.objectURI}
      placemark.delete()
      i += 1
    trackResult = {u'OK' : u'Track deleted: %s' % track.objectURI}
    track.delete()
    result = {'track' : trackResult, 'placemarks' : placemarksResult}
  except Track.DoesNotExist:
    result = {u'Error' : u'Track don\'t found: %s' % task['objectURI']}
  try:
    serverSyncTask = SyncTask.objects.get(objectURI=task['objectURI'], user=user)
    serverSyncTask.delete()
  except SyncTask.DoesNotExist:
    serverSyncTask = None
  return result