# -*- coding:utf-8 -*-

from django.db import models
from django.contrib.auth.models import User
from utils import formatTime

# Create your models here.

class Category(models.Model):
  user = models.ForeignKey(User)
  objectURI = models.CharField(max_length=255)
  name = models.CharField(max_length=100)
  updated = models.DateTimeField()
  
  class Meta:
    verbose_name_plural = u'categories'
  
  def __unicode__(self):
    return u'%s (%s) - %s' % (self.objectURI, self.user.username, self.name)

class Track(models.Model):
  user = models.ForeignKey(User)
  objectURI = models.CharField(max_length=255)
  category = models.ForeignKey(Category)
  name = models.CharField(max_length=100)
  date = models.DateTimeField()
  time = models.FloatField()
  distance = models.FloatField()
  minAltitude = models.FloatField()
  maxAltitude = models.FloatField()
  maxSpeed = models.FloatField()
  ppm = models.IntegerField(null=True, blank=True)
  note = models.TextField(null=True, blank=True)
  dataKML = models.TextField()
  updated = models.DateTimeField()
  
  def __unicode__(self):
    return u'%s (%s) - %s' % (self.objectURI, self.user.username, self.name)
  
  def distanceInKm(self):
    return self.distance / 1000
  
  def speedInKmH(self):
    return (self.distanceInKm() / (self.time / 3600))
  
  def maxSpeedInKmH(self):
    return (self.maxSpeed * 3.6)
  
  def pace(self):
    if (self.distance == 0):
      result = 0
    else:
      result = (self.time / 60) / (self.distanceInKm())
    return result
  
  def paceString(self):
    return formatTime(self.pace() * 60)
  
  def formattedTime(self):
    return formatTime(self.time)
    

class Placemark(models.Model):
  user = models.ForeignKey(User)
  track = models.ForeignKey(Track)
  objectURI = models.CharField(max_length=255)
  name = models.CharField(max_length=100)
  time = models.FloatField()
  distance = models.FloatField()
  startTime = models.DateTimeField()
  minAltitude = models.FloatField()
  maxAltitude = models.FloatField()
  maxSpeed = models.FloatField()
  
  def __unicode__(self):
    return u'%s (%s) - %s' % (self.objectURI, self.user.username, self.name)
  
  def speedInKmH(self):
    if self.distance == 0:
      result = 0
    else:
      result = (self.distance / 1000) / (self.time / 3600)
    return result
  
  def maxSpeedInKmH(self):
    return self.maxSpeed * 3.6
  
  def pace(self):
    if self.distance == 0:
      result = 0
    else:
      result = (self.time / 60) / (self.distance / 1000)
    return result
  
  def paceString(self):
    return formatTime(self.pace() * 60)
  
  def formattedTime(self):
    return formatTime(self.time)

class SyncTask(models.Model):
  user = models.ForeignKey(User)
  objectURI = models.CharField(max_length=255)
  typeTask = models.IntegerField()
  stateTask = models.IntegerField()
  
  def __unicode__(self):
    return u'%s (%s) - Type: %d - State: %d' % (self.objectURI, self.user.username, self.typeTask, self.stateTask)