# -*- coding:utf-8 -*-
from django.http import HttpResponse, HttpResponseServerError, HttpResponseRedirect, Http404
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404
from django.utils import simplejson as json
from django.utils.translation import ugettext as _
from django.contrib.messages import constants as messages
from django.contrib import auth
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User, check_password
from django.contrib.auth.decorators import login_required, user_passes_test
from api import *
from models import SyncTask, Category, Track, Placemark
from forms import TrackForm

def register(request):
  if request.method == 'POST':
    form = UserCreationForm(request.POST)
    if form.is_valid():
      newUser = form.save()
      return HttpResponseRedirect('/')
  else:
    form = UserCreationForm()
  return render_to_response('login/register.html', {'form': form}, RequestContext(request))

@login_required
def logout(request):
  auth.logout(request)
  return HttpResponseRedirect('/')

#Index view
@login_required
def index(request):
  tracks = Track.objects.filter(user=request.user).order_by('-date')
  return render_to_response('track.html', {'tracks': tracks}, context_instance=RequestContext(request))

@login_required
def track(request, id):
  tracks = Track.objects.filter(user=request.user).order_by('-date')
  try:
    theTrack = Track.objects.get(id=id, user=request.user)
  except Track.DoesNotExist:
    theTrack = None
    request.user.message_set.create(message=_('Requested track doesn\'t found'))
  placemarks = Placemark.objects.filter(track=theTrack, user=request.user).order_by('startTime')
  if len(placemarks) == 1:
    placemarks = None
  return render_to_response('track.html', {'tracks': tracks, 'track':theTrack, 'placemarks': placemarks}, context_instance=RequestContext(request))

def getKML(request, id):
  try:
    theTrack = Track.objects.get(id=id)
  except Track.DoesNotExist:
    theTrack = None
  if theTrack:
    result = theTrack.dataKML
  else:
    result = u''
  response = HttpResponse(mimetype='application/georss+xml; charset=utf-8')
  response['Content-Disposition'] = 'attachment; filename=data.kml'
  response.write(result)
  return response

@login_required
def edit_track(request, id):
  tracks = Track.objects.filter(user=request.user).order_by('-updated')
  try:
    track = Track.objects.get(id=id, user=request.user)
  except Track.DoesNotExist:
    raise Http404
  track = get_object_or_404(Track, id=id, user=request.user)
  if request.method == 'POST':
    form = TrackForm(request.POST)
    if form.is_valid():
      track.name = form.cleaned_data['name']
      track.ppm = form.cleaned_data['ppm']
      if track.ppm == None:
        track.ppm = 0
      track.note = form.cleaned_data['note']
      track.save()
      try:
        syncTask = SyncTask.objects.get(user=request.user, objectURI=track.objectURI)
      except SyncTask.DoesNotExist:
        syncTask = SyncTask(user=request.user, objectURI=track.objectURI, typeTask=1, stateTask=0)
        syncTask.save()
      request.user.message_set.create(message=_('The data was saved successfuly'))
      return HttpResponseRedirect(reverse('core.views.track',args=(track.id,)))
  else:
    form = TrackForm(instance=track)
  return render_to_response('edit_track.html', {'tracks': tracks, 'form': form, 'track':track}, context_instance=RequestContext(request))

def delete_track(request, id):
  try:
    track = Track.objects.get(id=id, user=request.user)
  except Track.DoesNotExist:
    raise Http404
  placemarks = Placemark.objects.filter(track=track, user=request.user)
  for placemark in placemarks:
    placemark.delete()
  syncTask = SyncTask(user=request.user, objectURI=track.objectURI, typeTask=2, stateTask=0)
  syncTask.save()
  track.delete()
  request.user.message_set.create(message=_('The data was deleted successfuly'))
  return HttpResponseRedirect(reverse('core.views.index'))

def sync(request):
  data = json.loads(request.POST['data'])
  credentials = json.loads(data['credentials'])
  tasksToSend = {}
  dataToSend = {}
  user = None
  if credentials['nickname'] != u'':
    try:
      user = User.objects.get(username=credentials['nickname'])
    except User.DoesNotExist:
      user = None
  if (user != None):
    if  (check_password(credentials['key'],user.password)):
      result = 0
      errors = False
      tasks = json.loads(data['tasks'])
      tasksData = json.loads(data['tasksData'])
      cadena = {}
      i = 0
      for j in range(len(tasks)):
        task = json.loads(tasks[u'%d' % j])
        taskData = None
        if tasksData.has_key(task['objectURI']):
          taskData = json.loads(tasksData[task['objectURI']])
        message = processTaskFromUser(task, user, taskData)
        if message.has_key('Error'):
          errors = True
        cadena[i] = message
        i += 1
        if errors:
          result = -1
      # Prepara les tasques de sincronització
      syncTasks = SyncTask.objects.all()
      i = 0
      for syncTask in syncTasks:
        cleanTask = False
        if syncTask.typeTask != 2:
          if syncTask.stateTask == 0 or syncTask.stateTask == 3:
            if syncTask.objectURI.find('Category') >= 0:
              category = Category.objects.get(objectURI=syncTask.objectURI, user=user)
              data = {'name' : category.name}
            else:
              track = Track.objects.get(objectURI=syncTask.objectURI, user=user)
              data = {'name' : track.name,
                        'ppm' : track.ppm,
                        'note' : track.note,
                        'category' : track.category.objectURI}
            dataToSend[syncTask.objectURI] = data
            cleanTask = True
        else:
          cleanTask = True
        tasksToSend[i] = { 'objectURI' : syncTask.objectURI,
                            'type' : syncTask.typeTask,
                            'state' : syncTask.stateTask }
        if cleanTask:
          syncTask.delete()
        i += 1
    else:
      result = -1
      cadena = _('Incorrect password.')
  else:
    result = -1
    cadena = _('User doesn\'t exist.')
  returnResult = {'resultCode': result, 'log' : cadena}
  if tasksToSend:
    returnResult['tasks'] = tasksToSend
  if dataToSend:
    returnResult['data'] = dataToSend
  return HttpResponse(json.dumps(returnResult), mimetype='application/json; charset=utf-8')